local addonName = ...
local guildName = nil
local db = nil
local guildMaster = nil

local today = nil
local sessionTotal = 0

------ Locale
local L_SESSION = "This session"
local L_NOINCOME = "You haven't generated any income to your guild for the past month - stop slacking!"
local L_NOINCOME_GM = "You haven't generated any income to your guild for the past month - stop slacking! %s is very disappointed."
local L_GOODBOY = "Good boy. Once you provide some more income, daddy will give you something special."
local L_GOODBOY_GM = "Good boy. Once you provide some more income, daddy %s will give you something special."
------

local function updateTooltip(tt)
	tt:AddLine("Guild Money")
	local added = nil
	if sessionTotal > 0 then
		tt:AddDoubleLine("This session", GetCoinTextureString(sessionTotal), 1, 1, 1, 0, 1, 0)
		added = true
	end
	if db[today] and db[today] > 0 then
		tt:AddDoubleLine(HONOR_TODAY, GetCoinTextureString(db[today]), 1, 1, 1, 0, 1, 0)
		added = true
	end
	local week = 0
	for i = today, (today - 7), -1 do
		week = week + (db[i] or 0)
	end
	if week > 0 then
		tt:AddDoubleLine(CALENDAR_REPEAT_WEEKLY, GetCoinTextureString(week), 1, 1, 1, 0, 1, 0)
		added = true
	end
	local month = week
	for i = (today - 7), (today - 30), -1 do
		month = month + (db[i] or 0)
	end
	if month > 0 then
		tt:AddDoubleLine(CALENDAR_REPEAT_MONTHLY, GetCoinTextureString(month), 1, 1, 1, 0, 1, 0)
		added = true
	end
	if not added then
		if guildMaster then
			tt:AddLine(L_NOINCOME_GM:format(guildMaster), 0.2, 1, 0.2, 1)
		else
			tt:AddLine(L_NOINCOME, 0.2, 1, 0.2, 1)
		end
	else
		tt:AddLine(" ")
		if guildMaster then
			tt:AddLine(L_GOODBOY_GM:format(guildMaster), 0.2, 1, 0.2, 1)
		else
			tt:AddLine(L_GOODBOY, 0.2, 1, 0.2, 1)
		end
	end
end

local ldb = LibStub("LibDataBroker-1.1")
local bGuildMoney = ldb:NewDataObject("Reagents", {
	type = "data source",
	icon = "Interface\\MoneyFrame\\UI-GoldIcon",
	text = "Guild money",
	OnTooltipShow = updateTooltip,
})

local matchGold = GOLD_AMOUNT:gsub("%%d", "(%%d+)")
local matchSilver = SILVER_AMOUNT:gsub("%%d", "(%%d+)")
local matchCopper = COPPER_AMOUNT:gsub("%%d", "(%%d+)")
local personalMoney = YOU_LOOT_MONEY:gsub("%%s", "(.+)")
local guildMoney = YOU_LOOT_MONEY_GUILD:gsub("%(", "%%("):gsub("%)", "%%)"):gsub("%%s", "(.+)")

local function getTotalFromString(msg)
	local g = msg:match(matchGold) or 0 * 1000
	local s = msg:match(matchSilver) or 0 * 100
	local c = msg:match(matchCopper) or 0
	return g + s + c
end

local function scanForGM()
	local p = UnitName("player")
	for i = 1, GetNumGuildMembers(true) do
		local n, _, r = GetGuildRosterInfo(i)
		if n and r == 0 then
			if n ~= p then guildMaster = n end
			break
		end
	end
end

local f = CreateFrame("Frame")
f:RegisterEvent("ADDON_LOADED")
f:RegisterEvent("GUILD_ROSTER_UPDATE")
f:SetScript("OnEvent", function(self, event, addon)
	if event == "ADDON_LOADED" then
		if addon ~= addonName then return end
		guildName = GetGuildInfo("player")
		if not guildName then
			print("startup failed.")
			return
		end
		BGMDB = BGMDB or {}
		if not BGMDB[guildName] then BGMDB[guildName] = {} end
		db = BGMDB[guildName]

		-- Borrowed from MoneyFu (probably ckk)
		local serverHour, serverMinute = GetGameTime()
		local utcHour = tonumber(date("!%H"))
		local utcMinute = tonumber(date("!%M"))
		local ser = serverHour + serverMinute / 60
		local utc = utcHour + utcMinute / 60
		local offset = floor((ser - utc) * 2 + 0.5) / 2
		if offset >= 12 then
			offset = offset - 24
		elseif offset < -12 then
			offset = offset + 24
		end
		today = floor((time() / 60 / 60 + offset) / 24)
		scanForGM()
	elseif event == "GUILD_ROSTER_UPDATE" then
		scanForGM()
	end
end)

local goldString = GOLD_AMOUNT:gsub("%%d", ""):trim()
local silverString = SILVER_AMOUNT:gsub("%%d", ""):trim()
local copperString = COPPER_AMOUNT:gsub("%%d", ""):trim()

local function filter(self, event, msg, ...)
	local personal, guild = msg:match(guildMoney)
	-- |not db| catches failed startup
	if not db or not guild then return end

	local looted = (getTotalFromString(guild) or 0)
	sessionTotal = sessionTotal + looted
	db[today] = (db[today] or 0) + looted

	local text = GetCoinText(sessionTotal, "")
	text = text:gsub(goldString, "g"):gsub(silverString, "s"):gsub(copperString, "c")
	bGuildMoney.text = text

	local t = GetCoinTextureString(getTotalFromString(personal))
	return false, YOU_LOOT_MONEY:format(t), ...
end

ChatFrame_AddMessageEventFilter("CHAT_MSG_MONEY", filter)

